package cz.muni.fi.pa165.dao;


import cz.muni.fi.pa165.entity.BaseEntity;

import java.util.List;
import java.util.UUID;

/**
 * Generic DAO interface
 * Provides CRUD operations
 *
 * @author Dominik Kozubík
 */
public interface BaseDao <Entity extends BaseEntity> {

    /**
     * Creates a persistent object
     *
     * @param t object to persist
     * @throws cz.muni.fi.pa165.dao.exceptions.DaoDataAccessException if any exception occurs
     */
    void create(Entity t);

    /**
     * Finds all persisted objects
     *
     * @return list of all persisted objects
     * @throws cz.muni.fi.pa165.dao.exceptions.DaoDataAccessException if any exception occurs
     */
    List<Entity> findAll();

    /**
     * Finds persisted object by id
     *
     * @param id id of persisted object
     * @throws cz.muni.fi.pa165.dao.exceptions.DaoDataAccessException if any exception occurs
     * @return object by given id
     */
    Entity findById(UUID id);

    /**
     * Updates persisted object
     *
     * @param t persisted object
     * @throws cz.muni.fi.pa165.dao.exceptions.DaoDataAccessException if any exception occurs
     * @return id of persisted object
     */
    UUID update(Entity t);

    /**
     * Removes persisted object
     *
     * @param t persisted object
     * @throws cz.muni.fi.pa165.dao.exceptions.DaoDataAccessException if any exception occurs
     */
    void remove(Entity t);
}
