package cz.muni.fi.pa165.dao;

import cz.muni.fi.pa165.dao.exceptions.DaoDataAccessException;
import cz.muni.fi.pa165.entity.BaseEntity;
import org.springframework.data.repository.NoRepositoryBean;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;
import java.util.UUID;

/**
 * @author Dominik Kozubík
 */
@NoRepositoryBean
public class BaseDaoImpl<Entity extends BaseEntity> implements BaseDao<Entity> {

    final Class<Entity> entityClass;

    @PersistenceContext
    protected EntityManager em;

    public BaseDaoImpl(Class<Entity> entityClass) {
        this.entityClass = entityClass;
    }

    @Override
    public void create(Entity entity) {
        try {
            em.persist(entity);
        } catch (Exception e) {
            throw new DaoDataAccessException(e.getMessage(), e);
        }
    }

    @Override
    public List<Entity> findAll() {
        try {
            return em.createQuery("select e from " + entityClass.getName() + " e", entityClass)
                    .getResultList();
        } catch (Exception e) {
            throw new DaoDataAccessException(e.getMessage(), e);
        }

    }

    @Override
    public Entity findById(UUID id) {
        try {
            return em.find(entityClass, id);
        } catch (Exception e)  {
            throw new DaoDataAccessException(e.getMessage(), e);
        }
    }

    @Override
    public UUID update(Entity t) {
        try {
            return em.merge(t).getId();
        } catch (Exception e) {
            throw new DaoDataAccessException(e.getMessage(), e);
        }
    }

    @Override
    public void remove(Entity entity) {
        try {
            var attached = findById(entity.getId());
            if (attached == null) {
                throw new IllegalArgumentException("Cannot delete a record. Given entity does not exist.");
            }
            em.remove(entity);
        } catch (Exception e) {
            throw new DaoDataAccessException(e.getMessage(), e);
        }
    }
}
