package cz.muni.fi.pa165.dao;

import cz.muni.fi.pa165.PersistenceSampleApplicationContext;
import cz.muni.fi.pa165.dao.person.PersonDao;
import cz.muni.fi.pa165.dao.player.PlayerDao;
import cz.muni.fi.pa165.entity.PersonEntity;
import cz.muni.fi.pa165.entity.PlayerEntity;
import cz.muni.fi.pa165.enums.CountryEnum;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.time.LocalDate;
import java.util.UUID;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * @author Adrian Piacek
 */
@ContextConfiguration(classes = PersistenceSampleApplicationContext.class)
@ExtendWith(SpringExtension.class)
@Transactional
public class PersonDaoTest {
    @Autowired
    private PersonDao personDao;

    EntityManagerFactory managerFactory = Persistence.createEntityManagerFactory("default");

    @AfterEach
    void afterEach() {
        EntityManager manager = null;
        try {
            manager = managerFactory.createEntityManager();
            manager.getTransaction().begin();
            manager.createQuery("delete from PersonEntity ").executeUpdate();
            manager.getTransaction().commit();
        } finally {
            if (manager != null) {
                manager.close();
            }
        }
    }

    @Test
    void create() {
        PersonEntity person = new PersonEntity("Johan");
        personDao.create(person);
        assertThat(person.getId()).isNotNull();
        assertThat(personDao.findAll()).hasSize(1);
    }

    @Test
    void findByIdExisting() {
        PersonEntity person = new PersonEntity("Johan");
        personDao.create(person);

        var find = personDao.findById(person.getId());
        assertThat(find).isEqualTo(person);
    }

    @Test
    void findByIdNonExisting() {
        assertThat(personDao.findAll()).isEmpty();

        var find = personDao.findById(UUID.randomUUID());
        assertThat(find).isNull();
    }

    @Test
    void findAll() {
        PersonEntity personA = new PersonEntity("PersonA");
        PersonEntity personB = new PersonEntity("PersonB");

        personDao.create(personA);
        personDao.create(personB);
        assertThat(personDao.findAll()).hasSize(2);
    }

    @Test
    void update() {
        PersonEntity personA = new PersonEntity("PersonA");
        personDao.create(personA);
        personA.setCountry(CountryEnum.AZERBAIJAN);
        personA.setDateOfBirth(LocalDate.of(2001,1,5));

        personDao.update(personA);
        assertThat(personDao.findById(personA.getId())).isEqualTo(personA);
    }

    @Test
    void delete() {
        PersonEntity personA = new PersonEntity("PersonA");
        PersonEntity personB = new PersonEntity("PersonB");
        personDao.create(personA);
        personDao.create(personB);
        personDao.remove(personA);
        var persons = personDao.findAll();
        assertThat(persons).hasSize(1).contains(personB);
    }
}
