package cz.muni.fi.pa165.dao;

import cz.muni.fi.pa165.PersistenceSampleApplicationContext;
import cz.muni.fi.pa165.dao.team.TeamDao;
import cz.muni.fi.pa165.entity.PlayerEntity;
import cz.muni.fi.pa165.entity.TeamCoachEntity;
import cz.muni.fi.pa165.entity.TeamEntity;
import cz.muni.fi.pa165.enums.CountryEnum;
import org.assertj.core.api.Assertions;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;

/**
 * @author Adrian Piacek
 */
@ContextConfiguration(classes = PersistenceSampleApplicationContext.class)
@ExtendWith(SpringExtension.class)
@Transactional
public class TeamDaoTest {

    @Autowired
    private TeamDao teamDao;

    EntityManagerFactory managerFactory = Persistence.createEntityManagerFactory("default");

    @AfterEach
    void afterEach(){
        EntityManager manager = null;
        try{
            manager = managerFactory.createEntityManager();
            manager.getTransaction().begin();
            manager.createQuery("delete from TeamEntity").executeUpdate();
            manager.getTransaction().commit();
        }finally {
            if (manager != null){
                manager.close();
            }
        }
    }

    @Test
    void create() {
        TeamCoachEntity teamCoachA = new TeamCoachEntity("AutistaLeader");
        TeamEntity team = new TeamEntity("Autisti", new HashSet<>(List.of(teamCoachA)));
        teamDao.create(team);
        assertThat(team.getId()).isNotNull();
        Assertions.assertThat(teamDao.findAll()).hasSize(1);
    }

    @Test
    void findByIdExisting() {
        TeamCoachEntity teamCoachA = new TeamCoachEntity("AutistaLeader");
        TeamEntity team = new TeamEntity("Autisti", new HashSet<>(List.of(teamCoachA)));
        teamDao.create(team);

        var find = teamDao.findById(team.getId());
        assertThat(find).isEqualTo(team);
    }

    @Test
    void findByIdNonExisting() {
        Assertions.assertThat(teamDao.findAll()).isEmpty();

        var find = teamDao.findById(UUID.randomUUID());
        assertThat(find).isNull();
    }

    @Test
    void findAll() {
        TeamCoachEntity teamCoachA = new TeamCoachEntity("AutistaA");
        TeamEntity teamA = new TeamEntity("Autisti I", new HashSet<>(List.of(teamCoachA)));
        TeamCoachEntity teamCoachB = new TeamCoachEntity("AutistaB");
        TeamEntity teamB = new TeamEntity("Autisti II", new HashSet<>(List.of(teamCoachB)));
        teamDao.create(teamA);
        teamDao.create(teamB);

        Assertions.assertThat(teamDao.findAll()).hasSize(2);
    }

    @Test
    void update() {
        TeamEntity team = new TeamEntity();
        teamDao.create(team);
        team.setName("AutistaC");
        team.setCountry(CountryEnum.BAHAMAS);
        teamDao.update(team);
        Assertions.assertThat(teamDao.findById(team.getId())).isEqualTo(team);
    }

    @Test
    void delete() {
        TeamCoachEntity teamCoachA = new TeamCoachEntity("AutistaA");
        TeamEntity teamA = new TeamEntity("Autisti I", new HashSet<>(List.of(teamCoachA)));
        TeamCoachEntity teamCoachB = new TeamCoachEntity("AutistaB");
        TeamEntity teamB = new TeamEntity("Autisti II", new HashSet<>(List.of(teamCoachB)));
        teamDao.create(teamA);
        teamDao.create(teamB);
        teamDao.remove(teamA);

        var teams = teamDao.findAll();
        Assertions.assertThat(teams).hasSize(1).contains(teamB);
    }


}
