package cz.muni.fi.pa165.dao;

import cz.muni.fi.pa165.PersistenceSampleApplicationContext;
import cz.muni.fi.pa165.dao.tournament.TournamentDao;
import cz.muni.fi.pa165.entity.TournamentEntity;
import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.enums.TournamentFormat;
import org.assertj.core.api.Assertions;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.transaction.Transactional;
import java.util.UUID;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;

/**
 * @author Adrian Piacek
 */
@ContextConfiguration(classes = PersistenceSampleApplicationContext.class)
@ExtendWith(SpringExtension.class)
@Transactional
public class TournamentDaoTest {

    @Autowired
    private TournamentDao tournamentDao;

    EntityManagerFactory managerFactory = Persistence.createEntityManagerFactory("default");

    @AfterEach
    void afterEach(){
        EntityManager manager = null;
        try{
            manager = managerFactory.createEntityManager();
            manager.getTransaction().begin();
            manager.createQuery("delete from TournamentEntity").executeUpdate();
            manager.getTransaction().commit();
        }finally {
            if (manager != null){
                manager.close();
            }
        }
    }

    @Test
    void create() {
        TournamentEntity tournament = new TournamentEntity("Prievidza", CountryEnum.SLOVAKIA, 1, TournamentFormat.DEFAULT, 3);
        tournamentDao.create(tournament);
        assertThat(tournament.getId()).isNotNull();
        Assertions.assertThat(tournamentDao.findAll()).hasSize(1);
    }

    @Test
    void findByIdExisting() {
        TournamentEntity tournament = new TournamentEntity("Prievidza", CountryEnum.SLOVAKIA, 1, TournamentFormat.DEFAULT, 3);
        tournamentDao.create(tournament);
        var find = tournamentDao.findById(tournament.getId());
        assertThat(find).isEqualTo(tournament);
    }

    @Test
    void findByIdNonExisting() {
        Assertions.assertThat(tournamentDao.findAll()).isEmpty();

        var find = tournamentDao.findById(UUID.randomUUID());
        assertThat(find).isNull();
    }

    @Test
    void findAll() {
        TournamentEntity tournamentA =
                new TournamentEntity("Prievidza", CountryEnum.SLOVAKIA, 1, TournamentFormat.DEFAULT, 3);
        TournamentEntity tournamentB =
                new TournamentEntity("Brno", CountryEnum.CZECH_REPUBLIC, 5, TournamentFormat.DEFAULT, 15);
        tournamentDao.create(tournamentA);
        tournamentDao.create(tournamentB);
        Assertions.assertThat(tournamentDao.findAll()).hasSize(2);
    }

    @Test
    void update() {
        TournamentEntity tournamentEntity = new TournamentEntity();
        tournamentDao.create(tournamentEntity);
        tournamentEntity.setCity("Bratislava");
        tournamentEntity.setPrizePool(25.0);
        tournamentEntity.setTeamCapacity(7);
        tournamentDao.update(tournamentEntity);
        Assertions.assertThat(tournamentDao.findById(tournamentEntity.getId())).isEqualTo(tournamentEntity);
    }

    @Test
    void delete() {
        TournamentEntity tournamentA =
                new TournamentEntity("Prievidza", CountryEnum.SLOVAKIA, 1, TournamentFormat.DEFAULT, 3);
        TournamentEntity tournamentB =
                new TournamentEntity("Brno", CountryEnum.CZECH_REPUBLIC, 5, TournamentFormat.DEFAULT, 15);
        tournamentDao.create(tournamentA);
        tournamentDao.create(tournamentB);
        tournamentDao.remove(tournamentA);
        var tournaments = tournamentDao.findAll();
        Assertions.assertThat(tournaments).hasSize(1).contains(tournamentB);
    }
}
