package cz.muni.fi.pa165.dao;

import cz.muni.fi.pa165.PersistenceSampleApplicationContext;
import cz.muni.fi.pa165.dao.tournamentOrganizer.TournamentOrganizerDao;
import cz.muni.fi.pa165.entity.PlayerEntity;
import cz.muni.fi.pa165.entity.TournamentOrganizerEntity;
import cz.muni.fi.pa165.enums.CountryEnum;
import org.assertj.core.api.Assertions;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.transaction.Transactional;
import java.util.UUID;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;

/**
 * @author Adrian Piacek
 */
@ContextConfiguration(classes = PersistenceSampleApplicationContext.class)
@ExtendWith(SpringExtension.class)
@Transactional
public class TournamentOrganizerDaoTest {

    @Autowired
    private TournamentOrganizerDao tournamentOrganizerDao;

    EntityManagerFactory managerFactory = Persistence.createEntityManagerFactory("default");

    @AfterEach
    void afterEach() {
        EntityManager manager = null;
        try {
            manager = managerFactory.createEntityManager();
            manager.getTransaction().begin();
            manager.createQuery("delete from TournamentOrganizerEntity").executeUpdate();
            manager.getTransaction().commit();
        } finally {
            if (manager != null) {
                manager.close();
            }
        }
    }

    @Test
    void create() {
        TournamentOrganizerEntity organizer = new TournamentOrganizerEntity("AutistaOrganizer");
        tournamentOrganizerDao.create(organizer);
        assertThat(organizer.getId()).isNotNull();
        Assertions.assertThat(tournamentOrganizerDao.findAll()).hasSize(1);
    }

    @Test
    void findByIdExisting() {
        TournamentOrganizerEntity organizer = new TournamentOrganizerEntity("AutistaOrganizer");
        tournamentOrganizerDao.create(organizer);
        var find = tournamentOrganizerDao.findById(organizer.getId());
        assertThat(find).isEqualTo(organizer);
    }

    @Test
    void findByIdNonExisting() {
        Assertions.assertThat(tournamentOrganizerDao.findAll()).isEmpty();

        var find = tournamentOrganizerDao.findById(UUID.randomUUID());
        assertThat(find).isNull();
    }

    @Test
    void findAll() {
        TournamentOrganizerEntity organizerA = new TournamentOrganizerEntity("AutistaOrganizerA");
        TournamentOrganizerEntity organizerB = new TournamentOrganizerEntity("AutistaOrganizerB");
        tournamentOrganizerDao.create(organizerA);
        tournamentOrganizerDao.create(organizerB);
        Assertions.assertThat(tournamentOrganizerDao.findAll()).hasSize(2);
    }

    @Test
    void update() {
        TournamentOrganizerEntity organizerEntity = new TournamentOrganizerEntity("PlayerA");
        tournamentOrganizerDao.create(organizerEntity);
        organizerEntity.setCountry(CountryEnum.DANZIG);
        organizerEntity.setName("NoPlayerA");
        tournamentOrganizerDao.update(organizerEntity);
        Assertions.assertThat(tournamentOrganizerDao.findById(organizerEntity.getId())).isEqualTo(organizerEntity);
    }

    @Test
    void delete() {
        TournamentOrganizerEntity organizerA = new TournamentOrganizerEntity("AutistaOrganizerA");
        TournamentOrganizerEntity organizerB = new TournamentOrganizerEntity("AutistaOrganizerB");
        tournamentOrganizerDao.create(organizerA);
        tournamentOrganizerDao.create(organizerB);
        tournamentOrganizerDao.remove(organizerA);
        var coaches = tournamentOrganizerDao.findAll();
        Assertions.assertThat(coaches).hasSize(1).contains(organizerB);
    }
}
