package cz.muni.fi.pa165.entity;

import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.enums.TournamentFormat;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.HashSet;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;

/**
 * @author Marek Kadlečík
 */
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class TeamEntityTest {

    private final EntityManagerFactory emf = Persistence.createEntityManagerFactory("default");

    private TeamEntity teamEntity;
    private TournamentEntity plannedTournament;
    private TournamentEntity completedTournament;
    private TeamCoachEntity teamCoach;
    private PlayerEntity playerA;

    @BeforeAll
    void init() {
        teamCoach = new TeamCoachEntity("Foo");
        teamEntity = new TeamEntity("Autisti III", new HashSet<>(List.of(teamCoach)));
        plannedTournament = new TournamentEntity("Prievidza", CountryEnum.SLOVAKIA, 1, TournamentFormat.DEFAULT, 3);
        completedTournament = new TournamentEntity("Osaka", CountryEnum.SLOVAKIA, 1000, TournamentFormat.DEFAULT, 10);
        playerA = new PlayerEntity("Bar");

        teamCoach.setTeam(teamEntity);
        teamEntity.setCountry(CountryEnum.ALBANIA);
        teamEntity.setPlannedTournaments(new HashSet<>(List.of(plannedTournament)));
        teamEntity.setCompletedTournaments(new HashSet<>(List.of(completedTournament)));
        plannedTournament.signTeamForTournament(teamEntity);
        completedTournament.addTeamToTournament(teamEntity);
        playerA.setTeam(teamEntity);
        teamEntity.addPlayerToTheTeam(playerA);

        EntityManager em = emf.createEntityManager();
        try {
            em.getTransaction().begin();

            em.persist(teamCoach);
            em.persist(playerA);
            em.persist(completedTournament);
            em.persist(plannedTournament);
            em.persist(teamEntity);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    /**
     * Checks attributes of team that were persisted into DB.
     */
    @Test
    void attributesTests() {
        TeamEntity team;

        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            team = em.find(TeamEntity.class, teamEntity.getId());
            assertEquals(team.getName(), "Autisti III");
            assertEquals(team.getCountry(), CountryEnum.ALBANIA);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @Test
    void relationshipTeam() {
        TeamEntity team;
        PlayerEntity player;
        TeamCoachEntity coach;
        EntityManager em1 = null;
        try {
            em1 = emf.createEntityManager();
            em1.getTransaction().begin();
            team = em1.find(TeamEntity.class, teamEntity.getId());
            coach = em1.find(TeamCoachEntity.class, teamCoach.getId());
            player = em1.find(PlayerEntity.class, playerA.getId());

            //https://github.com/typeorm/typeorm/issues/6582 Explanation for JoinColumn for player and coach.
            assertEquals(team.getCoachList(), new HashSet<>(List.of(teamCoach)));
            assertEquals(coach.getTeam(), teamEntity);
            assertEquals(player.getTeam(), teamEntity);

            assertEquals(team.getCompletedTournaments(), new HashSet<>(List.of(completedTournament)));
            assertEquals(team.getPlannedTournaments(), new HashSet<>(List.of(plannedTournament)));
            assertEquals(team.getPlayerList(), new HashSet<>(List.of(playerA)));

            em1.getTransaction().commit();
        } finally {
            if (em1 != null) em1.close();
        }
    }

    @AfterAll
    public void afterAll() {
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            em.createQuery("delete from PlayerEntity").executeUpdate();
            em.createQuery("delete from TeamCoachEntity").executeUpdate();
            em.createQuery("delete from TeamEntity").executeUpdate();
            em.getTransaction().commit();
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }
}
