package cz.muni.fi.pa165.services.teamCoach;

import cz.muni.fi.pa165.dao.teamCoach.TeamCoachDao;
import cz.muni.fi.pa165.dao.tournament.TournamentDao;
import cz.muni.fi.pa165.entity.PlayerEntity;
import cz.muni.fi.pa165.entity.TeamCoachEntity;
import cz.muni.fi.pa165.entity.TeamEntity;
import cz.muni.fi.pa165.entity.TournamentEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;
import java.util.UUID;

/**
 * @author Dominik Kozubík
 */
@Service
@Transactional
public class TeamCoachServiceImpl implements TeamCoachService {

    @Autowired
    private TeamCoachDao teamCoachDao;

    @Autowired
    private TournamentDao tournamentDao;

    @Override
    public void create(TeamCoachEntity teamCoach) {
        if (teamCoach == null) {
            throw new IllegalArgumentException("Given entity is null.");
        }
        teamCoachDao.create(teamCoach);
    }

    @Override
    public List<TeamCoachEntity> findAll() {
        return teamCoachDao.findAll();
    }

    @Override
    public TeamCoachEntity findById(UUID id) {
        if (id == null) {
            throw new IllegalArgumentException("Given id is null.");
        }
        return teamCoachDao.findById(id);
    }

    @Override
    public UUID update(TeamCoachEntity teamCoach) {
        if (teamCoach == null) {
            throw new IllegalArgumentException("Given entity is null.");
        }
        return teamCoachDao.update(teamCoach);
    }

    @Override
    public void remove(TeamCoachEntity teamCoach) {
        if (teamCoach == null) {
            throw new IllegalArgumentException("Given entity is null.");
        }
        teamCoachDao.remove(teamCoach);
    }

    @Override
    public void signForTournament(TeamCoachEntity teamCoach, TournamentEntity tournament) {

    }

    @Override
    public void addPlayerToTheTeam(TeamCoachEntity teamCoach, PlayerEntity newPlayer) {
        if (teamCoach == null || newPlayer == null) {
            throw new IllegalArgumentException("Given argument(s) is null");
        }

        teamCoach.addPlayerToTheTeam(newPlayer);
    }

    @Override
    public void removePlayerFromTheTeam(TeamCoachEntity teamCoach, PlayerEntity oldPlayer) {
        if (teamCoach == null || oldPlayer == null) {
            throw new IllegalArgumentException("Given argument(s) is null");
        }

        teamCoach.getTeam().removePlayerFromTheTeam(oldPlayer);
    }

    @Override
    public void addCoachToTheTeam(TeamEntity team, TeamCoachEntity newCoach) {
        if (team == null || newCoach == null) {
            throw new IllegalArgumentException("Given argument(s) is null");
        }

        team.addCoachToTheTeam(newCoach);
    }

    @Override
    public void removeCoachFromTheTeam(TeamEntity team, TeamCoachEntity oldCoach) {
        if (team == null || oldCoach == null) {
            throw new IllegalArgumentException("Given argument(s) is null");
        }

        team.removeCoachFromTheTeam(oldCoach);
    }
}
